import base64
import json

import lxml.html as htmlparser
import regex
import yarl
from Cryptodome.Cipher import AES

GOGOANIME_SECRET = b"25716538522938396164662278833288"
GOGOANIME_IV = b"1285672985238393"
CUSTOM_PADDER = "\x08\x0e\x03\x08\t\x03\x04\t"


def get_quality(url_text):
    match = regex.search(r"(\d+) P", url_text)

    if not match:
        return None

    return int(match.group(1))


def pad(data):
    return data + CUSTOM_PADDER[(len(CUSTOM_PADDER) - len(data) % 16) :]


def aes_encrypt(data: str):
    return base64.b64encode(
        AES.new(GOGOANIME_SECRET, AES.MODE_CBC, iv=GOGOANIME_IV).encrypt(
            pad(data).encode()
        )
    )


def aes_decrypt(data: str):
    return AES.new(GOGOANIME_SECRET, AES.MODE_CBC, iv=GOGOANIME_IV).decrypt(
        base64.b64decode(data)
    )


def extract(session, url, **opts):
    """
    Extract content off of GogoAnime.
    """

    content_info = aes_decrypt(
        htmlparser.fromstring(session.get(url).text)
        .cssselect('[data-name="crypto"]')[0]
        .get("data-value")
    )
    content_id = content_info[: content_info.index(b"&")].decode()

    parsed_url = yarl.URL(url)
    next_host = "https://{}/".format(parsed_url.host)

    response = session.get(
        "{}encrypt-ajax.php".format(next_host),
        params={"id": aes_encrypt(content_id).decode()},
        headers={"x-requested-with": "XMLHttpRequest"},
    )
    content = json.loads(
        aes_decrypt(response.json().get("data"))
        .replace(b'o"<P{#meme":', b'e":[{"file":')
        .decode("utf-8")
        .strip("\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10")
    )

    def yielder():
        for origin in content.get("source"):
            yield {
                "stream_url": origin.get("file"),
                "quality": get_quality(origin.get("label", "")),
                "headers": {"referer": next_host},
            }

        for backups in content.get("source_bk"):
            yield {
                "stream_url": backups.get("file"),
                "quality": get_quality(origin.get("label", "")),
                "headers": {"referer": next_host},
            }

    return list(yielder())



from urllib.parse import unquote, quote
from textwrap import wrap
import base64

NORMAL_TABLE = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
BASE64_TABLE = "0wMrYU+ixjJ4QdzgfN2HlyIVAt3sBOZnCT9Lm7uFDovkb/EaKpRWhqXS5168ePcG"


def encrypt_to_four(plaintext):

    cipher = ""

    for c in range(0, len(plaintext), 3):
        mapping = [ord(plaintext[c]) >> 2, 3 & ord(plaintext[c]) << 4, None, None]

        if len(plaintext) > (c + 1):
            mapping[1] |= ord(plaintext[c + 1]) >> 4
            mapping[2] = (15 & ord(plaintext[c + 1])) << 2

        if len(plaintext) > (c + 2):
            mapping[2] = (mapping[2] or 0) | ord(plaintext[c + 2]) >> 6
            mapping[3] = 63 & ord(plaintext[c + 2])

        for data in mapping:
            if data is None:
                cipher += "="
            else:
                if 0 <= data and data < 64:
                    cipher += BASE64_TABLE[data]

        return cipher


def cipher_keyed(key, plaintext):
    cipher = ""
    xcrypto = 0

    mapping = {_: _ for _ in range(256)}

    for c in range(256):
        xcrypto = (xcrypto + mapping[c] + ord(key[c % len(key)])) % 256
        mapping[c], mapping[xcrypto] = mapping[xcrypto], mapping[c]

    i = j = 0
    for f in range(0, len(plaintext)):
        j = (j + f) % 256
        i = (i + mapping[j]) % 256
        mapping[i], mapping[j] = mapping[j], mapping[i]
        cipher += chr(ord(plaintext[f]) ^ mapping[(mapping[i] + mapping[j]) % 256])

    return cipher


def get_salted_code(plaintext):
    part_1 = encrypt_to_four((unquote(plaintext) + "000000")[:6])[::-1]
    return part_1 + encrypt_to_four(cipher_keyed(part_1, unquote(plaintext))).replace(
        "=", ""
    )


def encrypt(data):
    return "".join(
        BASE64_TABLE[
            int(segment.ljust(6, "0"), 2) if len(segment) < 6 else int(segment, 2)
        ]
        for segment in wrap(
            "".join(bin(ord(c)).lstrip("0b").rjust(8, "0") for c in data), 6
        )
    )


def decrypt_url(encrypted_url, n=6):
    return unquote(cipher_keyed(encrypted_url[:n], decrypt(encrypted_url[n:])))


def encrypt_url(url):
    return "kr1337" + encrypt(cipher_keyed("kr1337", quote(url)))


def decrypt(data):
    return "".join(
        chr(_)
        for _ in base64.b64decode(
            data.translate(str.maketrans(BASE64_TABLE, NORMAL_TABLE)).encode()
        )
    )
